#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <GL/glut.h>
#include "image.h"

struct image {
	unsigned int tex;
	int xsz, ysz;
	int txsz, tysz;
};

void redraw(void);
void reshape(int x, int y);
void keyb(unsigned char key, int x, int y);
static void set_active_texture(int img);
static int next_pow2(int x);

struct image *images;
int imgnum, actimg;

int main(int argc, char **argv)
{
	int i;

	if(!(images = malloc((argc - 1) * sizeof *images))) {
		perror("malloc failed");
		return 1;
	}

	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);
	glutInitWindowSize(300, 200);
	glutCreateWindow("imgtest");
	glutDisplayFunc(redraw);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyb);

	for(i=1; i<argc; i++) {
		void *pixels;
		struct image *img = &images[imgnum];

		if(!(pixels = ImgLoadImage(argv[i], (unsigned int*)&img->xsz, (unsigned int*)&img->ysz))) {
			fprintf(stderr, "failed to load image: %s\n", argv[i]);
			continue;
		}

		if(strstr((char*)glGetString(GL_EXTENSIONS), "ARB_texture_non_power_of_two")) {
			img->txsz = img->xsz;
			img->tysz = img->ysz;
		} else {
			img->txsz = next_pow2(img->xsz);
			img->tysz = next_pow2(img->ysz);
		}

		glGenTextures(1, &img->tex);
		if(glGetError() != GL_NO_ERROR) {
			fprintf(stderr, "failed to create an OpenGL texture object\n");
			return 1;
		}
		glBindTexture(GL_TEXTURE_2D, img->tex);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		glTexImage2D(GL_TEXTURE_2D, 0, 4, img->txsz, img->tysz, 0, GL_BGRA, GL_UNSIGNED_BYTE, pixels);
		if(glGetError() != GL_NO_ERROR) {
			fprintf(stderr, "failed to make an OpenGL texture for: %s\n", argv[i]);
			glDeleteTextures(1, &img->tex);
			continue;
		}

		ImgFreeImage(pixels);
		imgnum++;
	}

	if(!imgnum) {
		return 1;
	}

	set_active_texture(0);
	glEnable(GL_TEXTURE_2D);
	glClearColor(0, 0, 0, 0);

	glutMainLoop();
	return 0;
}

void redraw(void)
{
	glClear(GL_COLOR_BUFFER_BIT);

	glBegin(GL_QUADS);
	glTexCoord2f(0, 1); glVertex2f(-1, -1);
	glTexCoord2f(1, 1); glVertex2f(1, -1);
	glTexCoord2f(1, 0); glVertex2f(1, 1);
	glTexCoord2f(0, 0); glVertex2f(-1, 1);
	glEnd();

	glutSwapBuffers();
}

void reshape(int x, int y)
{
	glViewport(0, 0, x, y);
}

void keyb(unsigned char key, int x, int y)
{
	switch(key) {
	case ' ':
		set_active_texture((actimg + 1) % imgnum);
		glutPostRedisplay();
		break;

	case 27:
		exit(0);

	default:
		break;
	}
}

static void set_active_texture(int img)
{
	if(img < imgnum) {
		actimg = img;
		glutReshapeWindow(images[actimg].xsz, images[actimg].ysz);
		glBindTexture(GL_TEXTURE_2D, images[actimg].tex);

		glMatrixMode(GL_TEXTURE);
		glLoadIdentity();
		glScalef((float)images[actimg].xsz / (float)images[actimg].txsz, (float)images[actimg].ysz / (float)images[actimg].tysz, 1);
	}
}

static int next_pow2(int x)
{
	float fx = (float)x;
	int exp = 0;
	while(fx > 1.0) {
		fx /= 2.0;
		exp++;
	}
	return (int)pow(2, exp);
}

